<?php if ( ! defined( 'ABSPATH' ) ) exit;


function woocommerce_accounting_export_data() {
    global $woocommerce, $post;

    //test nonce
    if (!isset( $_POST['_check_export'] ) || ! wp_verify_nonce( $_POST['_check_export'], 'check_nonce_export')){
        wp_die('Sorry, your nonce did not verify.');
        exit;
    }


    $separator = sanitize_text_field($_POST['woocommerce_accounting_export_separator']);
    $dec_separator = sanitize_text_field($_POST['woocommerce_accounting_export_decimal_separator']);
    $ts1 = sanitize_text_field($_POST['woocommerce_accounting_export_start_date']);
    $ts2 = sanitize_text_field($_POST['woocommerce_accounting_export_end_date']);
    $date_opt = sanitize_text_field($_POST['woocommerce_accounting_export_date_opt']);
    $date_format_opt = sanitize_text_field($_POST['woocommerce_accounting_export_date_format']);
    $cust_opt = sanitize_text_field($_POST['woocommerce_accounting_export_cust_opt']);
    $pay_opt = sanitize_text_field($_POST['woocommerce_accounting_export_pay_opt']);
    $prod_opt = sanitize_text_field($_POST['woocommerce_accounting_export_prod_opt']);
    $taxes_opt = sanitize_text_field($_POST['woocommerce_accounting_export_taxes_opt']);
    $ship_opt = sanitize_text_field($_POST['woocommerce_accounting_export_ship_opt']);
    $factnum_opt = sanitize_text_field($_POST['woocommerce_accounting_export_factnum_opt']);
    $zero_opt = sanitize_text_field($_POST['woocommerce_accounting_export_zero_opt']);

    update_option( 'woocommerce_accounting_export_start_date',$ts1);
    update_option( 'woocommerce_accounting_export_end_date',$ts2);
    update_option( 'woocommerce_accounting_export_date_format',$date_format_opt);
    update_option( 'woocommerce_accounting_export_separator',$separator);
    update_option( 'woocommerce_accounting_export_date_opt',$date_opt);
    update_option( 'woocommerce_accounting_export_pay_opt',$pay_opt);
    update_option( 'woocommerce_accounting_export_prod_opt',$prod_opt);
    update_option( 'woocommerce_accounting_export_taxes_opt',$taxes_opt);
    update_option( 'woocommerce_accounting_export_ship_opt',$ship_opt);
    update_option( 'woocommerce_accounting_export_factnum_opt',$factnum_opt);
    update_option( 'woocommerce_accounting_export_cust_opt',$cust_opt);
    update_option( 'woocommerce_accounting_export_decimal_separator',$dec_separator);
    update_option( 'woocommerce_accounting_export_zero_opt',$zero_opt);

    if ($separator == "t") {
        $separator = "\t";
    }

    $order_status = get_option('woocommerce_accounting_status');

    $order_args = array(
        'posts_per_page' => -1,
        'post_type' => 'shop_order',
        'post_status' => $order_status,
        'orderby' => 'ID',
        'order' => 'ASC',
    );

    if ($date_opt == "1") {
        $order_args['meta_query'] = array(
            array(
                'key' => '_completed_date',
                'value' => array($ts1, $ts2),
                'compare' => 'BETWEEN',
                'type' => 'DATE',
            )
        );
    }
    elseif($date_opt == "0"){
        $order_args['date_query'] = array(
            array(
                'before' => $ts2,
                'after' => $ts1,
                'inclusive' => true
            )
        );
    }
    elseif ($date_opt == "2") {
        $order_args['meta_query'] = array(
            array(
                'key' => '_wcpdf_invoice_date',
                'value' => array($ts1, $ts2),
                'compare' => 'BETWEEN',
                'type' => 'DATE'
            )
        );
    }
    elseif ($date_opt == "3") {
        $order_args['meta_query'] = array(
            array(
                'key' => '_paid_date',
                'value' => array($ts1, $ts2),
                'compare' => 'BETWEEN',
                'type' => 'DATE'
            )
        );
    }
    $orders = get_posts($order_args);
    do_action('woocommerce_accounting:export:launch', $orders);


    //Collecting datas
    if (!empty($orders)) {
        //Generic accounts
        $csv_headers = get_option ('woocommerce_accounting_columns_headers');
        $book_code = get_option('woocommerce_accounting_book_code_order');
        $original_book_code = get_option('woocommerce_accounting_book_code_order');
        $book_code_status = get_option('woocommerce_accounting_status_code');
        $book_account_status = get_option('woocommerce_accounting_status_account');
        $gen_account_cust = get_option('woocommerce_accounting_generic_cust_accounting_account');
        $gen_account_prod = get_option ('woocommerce_accounting_generic_prod_accounting_account');
        $gen_account_fdp = get_option ('woocommerce_accounting_generic_fdp_accounting_account');
        $gen_account_tax = get_option ('woocommerce_accounting_generic_tax_accounting_account');
        $gen_analytic_prod = get_option ('woocommerce_accounting_generic_prod_analytic_account');
        $gen_analytic_fdp = get_option('woocommerce_accounting_generic_fdp_analytic_account');
        $gen_account_discount = get_option('woocommerce_accounting_generic_discount_accounting_account');
        $gen_analytic_discount = get_option('woocommerce_accounting_generic_discount_analytic_account');
        $gen_sku_discount = get_option('woocommerce_accounting_generic_discount_product_sku');

        foreach ($orders as $order) {

            if ($date_opt == "0") {
                $order->piecedate =  get_post_field( 'post_date', $order->ID );
            }
            elseif ($date_opt == "1"){
                $order->piecedate = get_post_meta($order->ID,'_completed_date',true);
            }
            elseif ($date_opt == "3"){
                $order->piecedate = get_post_meta($order->ID,'_paid_date',true);
            }
            else {
                $invoice_date = get_post_meta($order->ID,'_wcpdf_invoice_date',true);
                if (!empty($invoice_date)) {
                    $order->piecedate = $invoice_date;
                }
                else {
                    $order->piecedate = get_post_field( 'post_date', $order->ID );
                }
            }
            $order->original_date = $order->piecedate;
            // Test PDF Invoices and Packing slips activated

            if ( in_array( 'woocommerce-pdf-invoices-packing-slips/woocommerce-pdf-invoices-packingslips.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {

                $pdfoptions = get_option('wpo_wcpdf_template_settings');
                $invoice_number_data = get_post_meta($order->ID,'_wcpdf_invoice_number_data');
                $invoice_number = get_post_meta($order->ID,'_wcpdf_invoice_number',true);
                $invoice_date = get_post_meta($order->ID,'_wcpdf_invoice_date',true);

                //Invoice number presence test
                if ( ( !empty ($invoice_number) ) && ( empty ($invoice_number_data) ) && $factnum_opt == 0 ){
                    //Creating invoices numbers
                    $prefix = $pdfoptions['invoice_number_formatting_prefix'];
                    $suffix = $pdfoptions['invoice_number_formatting_suffix'];
                    $padding = $pdfoptions['invoice_number_formatting_padding'];

                    $order_year = date_i18n( 'Y', strtotime( $order->piecedate ) );
                    $order_month = date_i18n( 'm', strtotime( $order->piecedate ) );
                    $order_day = date_i18n( 'd', strtotime( $order->piecedate ) );
                    $invoice_year = date_i18n( 'Y', strtotime( $invoice_date ) );
                    $invoice_month = date_i18n( 'm', strtotime( $invoice_date ) );
                    $invoice_day = date_i18n( 'd', strtotime( $invoice_date ) );

                    $formats['prefix'] = isset($prefix)?$prefix:'';
                    $formats['suffix'] = isset($suffix)?$suffix:'';
                    $formats['padding'] = isset($padding)?$padding:'';

                    // Remplacement of invoices dates in invoices numbers
                    foreach ($formats as $key => $value) {
                        $value = str_replace('[order_year]', $order_year, $value);
                        $value = str_replace('[order_month]', $order_month, $value);
                        $value = str_replace('[order_day]', $order_day, $value);
                        $value = str_replace('[invoice_year]', $invoice_year, $value);
                        $value = str_replace('[invoice_month]', $invoice_month, $value);
                        $value = str_replace('[invoice_day]', $invoice_day, $value);
                        $formats[$key] = $value;
                    }

                    // Padding
                    if ( ctype_digit( (string)$formats['padding'] ) ) {
                        $invoice_number = sprintf('%0'.$formats['padding'].'d', $invoice_number);
                    }
                    $formatted_invoice_number = $formats['prefix'] . $invoice_number . $formats['suffix'] ;

                    //Invoice number OK
                    $order->number = $formatted_invoice_number;
                }
                else if ( ( !empty ($invoice_number) ) && $factnum_opt == 0 ){
                    $order->number = $invoice_number;
                }
                else {
                    $order->number = $order->ID;
                }

                //Invoice number not OK (order number)

            }
            else {
                $order->number = $order->ID;
            }

            //Customers exports options
            if ($cust_opt =="0") {
                  $order->custid = get_post_meta($order->ID,'_customer_user',true);
                  $order->account_cust = get_user_meta($order->custid,'woocommerce_accounting_user_accountingcode',true);
            }
            elseif ($cust_opt =="1"){
                $order->gatewayid = get_post_meta($order->ID,'_payment_method',true);
                if (!empty($order->gatewayid)) {
                    $order->gatewayid = get_post_meta($order->ID,'_payment_method',true);
                    $order_country_acc = get_option('woocommerce_accounting_gateways_libs');
                    $order->account_cust = is_array($order_country_acc) ? $order_country_acc[$order->gatewayid]['acc'] : '';
                }
                else {
                    $order->account_cust = get_user_meta($order->custid,'woocommerce_accounting_user_accountingcode',true);
                }
            }


            // Get the country only in label if prod by country
            if ($prod_opt == "2") {
                $country_label = get_post_meta($order->ID,'_shipping_country',true);
            }
            else {
                $country_label = "";
            }
            //Payment gateway indication in label
            if ($pay_opt == "1") {
                $pay_meth = substr(get_post_meta($order->ID,'_payment_method',true),0,1) . substr(get_post_meta($order->ID,'_payment_method',true), 3, 1) . substr(get_post_meta($order->ID,'_payment_method',true),-1) ;
                $order->lib = remove_accents (strtoupper($pay_meth) . ' ' .  get_option ('woocommerce_accounting_lib_prefix') . ' ' . strtoupper(get_post_meta($order->ID,'_billing_company',true)) . ' ' . ucfirst(get_post_meta($order->ID,'_billing_last_name',true)) . ' ' . ucfirst(get_post_meta($order->ID,'_billing_first_name',true)) . ' ' . $country_label);
                $order->lib = trim($order->lib);
            }
            elseif ($pay_opt == "2") {
                $order_pay_meth = get_post_meta($order->ID,'_payment_method',true);
                if (!empty (get_option('woocommerce_accounting_gateways_libs')[$order_pay_meth]['lib'])) {
                    $listed_pay_lib = get_option('woocommerce_accounting_gateways_libs')[$order_pay_meth]['lib'];
                    $pay_meth = $listed_pay_lib;
                }
                else {
                    $pay_meth = substr(get_post_meta($order->ID,'_payment_method',true),0,1) . substr(get_post_meta($order->ID,'_payment_method',true), 3, 1) . substr(get_post_meta($order->ID,'_payment_method',true),-1) ;
                };
                $order->lib = remove_accents (strtoupper($pay_meth) . ' ' .  get_option ('woocommerce_accounting_lib_prefix') . ' ' . strtoupper(get_post_meta($order->ID,'_billing_company',true)) . ' ' . ucfirst(get_post_meta($order->ID,'_billing_last_name',true)) . ' ' . ucfirst(get_post_meta($order->ID,'_billing_first_name',true)) . ' ' . $country_label);
                $order->lib = trim($order->lib);
            }
            else {
                $order->lib = remove_accents (get_option ('woocommerce_accounting_lib_prefix') . ' ' . strtoupper(get_post_meta($order->ID,'_billing_company',true)) . ' ' . ucfirst(get_post_meta($order->ID,'_billing_last_name',true)) . ' ' . ucfirst(get_post_meta($order->ID,'_billing_first_name',true)) . ' ' . $country_label);
                $order->lib = trim($order->lib);
            }

            if ( $factnum_opt == 1 ){

                if  ( (!empty ($invoice_number) ) && ( empty ($invoice_number_data) ) ) {
                    //Creating invoices numbers
                    $prefix = $pdfoptions['invoice_number_formatting_prefix'];
                    $suffix = $pdfoptions['invoice_number_formatting_suffix'];
                    $padding = $pdfoptions['invoice_number_formatting_padding'];

                    $order_year = date_i18n( 'Y', strtotime( $order->piecedate ) );
                    $order_month = date_i18n( 'm', strtotime( $order->piecedate ) );
                    $order_day = date_i18n( 'd', strtotime( $order->piecedate ) );
                    $invoice_year = date_i18n( 'Y', strtotime( $invoice_date ) );
                    $invoice_month = date_i18n( 'm', strtotime( $invoice_date ) );
                    $invoice_day = date_i18n( 'd', strtotime( $invoice_date ) );


                    $formats['prefix'] = isset($prefix)?$prefix:'';
                    $formats['suffix'] = isset($suffix)?$suffix:'';
                    $formats['padding'] = isset($padding)?$padding:'';

                    // Invoices dates replacements in invoices
                    foreach ($formats as $key => $value) {
                        $value = str_replace('[order_year]', $order_year, $value);
                        $value = str_replace('[order_month]', $order_month, $value);
                        $value = str_replace('[order_day]', $order_day, $value);
                        $value = str_replace('[invoice_year]', $invoice_year, $value);
                        $value = str_replace('[invoice_month]', $invoice_month, $value);
                        $value = str_replace('[invoice_day]', $invoice_day, $value);
                        $formats[$key] = $value;
                    }

                    // Padding
                    if ( ctype_digit( (string)$formats['padding'] ) && $formats['padding'] > 3 ) {
                        $invoice_number = sprintf('%0'.$formats['padding'].'d', $invoice_number);
                    }
                    $formatted_invoice_number = $formats['prefix'] . $invoice_number . $formats['suffix'] ;
                }
                elseif ( !empty ($invoice_number)  ) {
                    $formatted_invoice_number = $invoice_number;
                }

                if (isset($formatted_invoice_number)){
                    $order->lib = $formatted_invoice_number . ' ' . $order->lib;
                }

            }


            //Date Format
            $order->piecedate = woocommerce_accounting_date_format($order->piecedate, $date_format_opt);
            $order->date_format_opt = $date_format_opt;

            $order->outcome = round(get_post_meta($order->ID,'_order_total',true),2);
            $order->income_tax = round(get_post_meta($order->ID, '_order_tax',true),2) + round(get_post_meta($order->ID, '_order_shipping_tax',true),2);
            $order->income_fdpht = round(get_post_meta($order->ID, '_order_shipping',true),2);
            $order->income_prodht = ( ($order->outcome) - ( ($order->income_tax) + ($order->income_fdpht) ) );
            //WC pre 2.3 bug on coupons correcting to match legal statements

            $order->order_discount = get_post_meta($order->ID, '_order_discount',true);

            if ($order->order_discount > 0) {
                $order->order_ex_discount = $order->order_discount + $order->outcome;

                //Calculating
                $order->prodht_exdisc = $order->order_ex_discount - $order->income_tax - $order->income_fdpht ;
                //Shipment
                $quotient_fdp = $order->income_fdpht / $order->order_ex_discount;
                //Products
                $quotient_prod = $order->prodht_exdisc / $order->order_ex_discount;
                //Taxes
                $quotient_tax = $order->income_tax / $order->order_ex_discount;
                //Recalculate the right amounts
                //Products
                $order->income_prodht = $order->outcome * $quotient_prod ;
                //Shipment
                $order->income_fdpht = $order->outcome * $quotient_fdp ;
                //Taxes
                $order->income_tax = $order->outcome * $quotient_tax;
            }

            do_action_ref_array('woocommerce_accounting:export:order_values', array(&$order));
        } // End foreach

        $output = fopen("php://output",'w') or die ("Can't open php://output");
        //CSV Headers
        $filename = 'woocommerce-accounting-wbk-export-' . $ts1 . '-' . $ts2 . '.csv';
        header( 'Content-type: application/csv' );
        header( 'Content-Disposition: attachment; filename='.$filename );
        header('Content-Transfer-Encoding: UTF-8');
        header( 'Pragma: no-cache' );
        header( 'Expires: 0' );

        // CSV Columns headers
        $csv_journal = !empty($csv_headers['journal']) ? $csv_headers['journal'] : $csv_journal = 'Code_Journal';
        $csv_date = !empty($csv_headers['date']) ? $csv_headers['date'] : 'Date_de_piece';
        $csv_number = !empty($csv_headers['number']) ? $csv_headers['number'] : 'Numero_de_piece';
        $csv_code = !empty($csv_headers['code']) ? $csv_headers['code'] : 'Compte_Comptable';
        $csv_label = !empty($csv_headers['label']) ? $csv_headers['label'] : 'Libelle';
        $csv_outcome = !empty($csv_headers['outcome']) ? $csv_headers['outcome'] : 'Debit';
        $csv_income = !empty($csv_headers['income']) ? $csv_headers['income'] : 'Credit';
        $csv_center = !empty($csv_headers['center']) ? $csv_headers['center'] : 'Code_Analytique';

        $fillwith = $zero_opt == "1" ? 0.00 : '';

        $col_map = get_option('woocommerce_accounting_colorder');
        $original_book_code = get_option('woocommerce_accounting_book_code_order');

        //Index table to order columns
        $header_line = array(
            $col_map['Journal'] =>    $csv_journal,
            $col_map['Date'] =>    $csv_date,
            $col_map['Inv.Number'] =>    $csv_number,
            $col_map['Acc.Code'] =>    $csv_code,
            $col_map['Label'] =>    $csv_label,
            $col_map['Outcome'] =>    $csv_outcome,
            $col_map['Income'] =>    $csv_income,
            $col_map['Cost Center'] =>    $csv_center,
            $col_map['Empty 1'] =>    "",
            $col_map['Empty 2'] =>    "",
            $col_map['Empty 3'] =>    "",
            $col_map['Empty 4'] =>    ""
        );

        ksort($header_line);
        fputcsv ($output, $header_line, $separator);

        do_action_ref_array('woocommerce_accounting:export:after_headline', array(&$output, $col_map, $separator));

        foreach($orders as $order) {

            if(isset($order->bypass) && $order->bypass){
                do_action_ref_array('woocommerce_accounting:export:after_order_output', array(&$output, $order, $separator));
                continue;
            }

            $get_order = new WC_Order ( $order->ID );
            $post_status = get_post_status($order->ID);

            if(isset($book_code_status[$post_status]) && $book_code_status[$post_status] != ""){
                $book_code = $book_code_status[$post_status];
            }
            else{
                $book_code = $original_book_code;
            }
            /******************************************************************************/
            /************************ Ajout des taxes spécifiques **************************/
            /******************************************************************************/

            $tax_applied_check = (float)get_post_meta($order->ID,'_order_tax',true, 0) + (float)get_post_meta($order->ID,'_order_shipping_tax',true, 0);
            // Tax application test
            if (($tax_applied_check != 0) && ($taxes_opt == "1") && ($order->order_discount == 0)){
                // Finding the right order taxes
                $tax_displays = array ();
                foreach ($get_order->get_taxes() as $get_order_tax) {
                    $tax_rate_id = $get_order_tax['rate_id'];
                    $tax_line_amount = $get_order_tax['tax_amount'] + $get_order_tax['shipping_tax_amount'];
                    if($tax_line_amount==0){
                        continue;
                    }
                    // getting the taxe account if tax OK and tax account does exist
                    if ( (!empty(get_option('woocommerce_accounting_tax_rates_accounting_account')[$tax_rate_id])) && (!empty(get_option('woocommerce_accounting_tax_rates_accounting_account')[$tax_rate_id]['acc']) ) ) {
                        $tax_line_account = get_option ('woocommerce_accounting_tax_rates_accounting_account')[$tax_rate_id]['acc'];
                        $tax_displays [] = array (
                            'account' => $tax_line_account,
                            'amount' => $tax_line_amount
                        );
                    }
                    // else generic tax account and tax line total
                     else {
                        $tax_line_account = $gen_account_tax;
                        $tax_displays [] = array (
                            'account' => $tax_line_account,
                            'amount' => $tax_line_amount
                        );
                    }
                }
            }
            // else generic tax account and tax order total
            else {
                $tax_line_account = $gen_account_tax;
                $tax_line_amount = $order->income_tax;
            }

            /******************************************************************************/
            /********** Ajout des comptes comptables des méthodes d'expédition ************/
            /******************************************************************************/

            $order_shipping_methods = $get_order->get_shipping_methods( );

            foreach ($order_shipping_methods as $order_shipping_method){
                $order_shipping_acc_list = get_option ('woocommerce_accounting_shipping_methods_accounting_account');
                if ($ship_opt == "1") {
                    // Shipment account existence test or apply generic
                    if (!empty($order_shipping_acc_list[$order_shipping_method['method_id']]['acc'])) {
                        $order_shipping_accounting_acc = $order_shipping_acc_list[$order_shipping_method['method_id']]['acc'];
                    }
                    else {
                        $order_shipping_accounting_acc = $gen_account_fdp;
                    }

                    //Shipment analytic code existence test or apply generic
                    $order_shipping_ana_list = get_option ('woocommerce_accounting_shipping_methods_analytic_account');
                    if (!empty($order_shipping_ana_list[$order_shipping_method['method_id']]['ana'])) {
                        $order_shipping_analytic_acc = $order_shipping_ana_list[$order_shipping_method['method_id']]['ana'];
                    }
                    else {
                        $order_shipping_analytic_acc = $gen_analytic_fdp;
                    }
                }
                else {
                    $order_shipping_accounting_acc = $gen_account_fdp;
                    $order_shipping_analytic_acc = $gen_analytic_fdp;
                }
            }

            /******************************************************************************/
            /*************** Ajout des comptes comptables par produits. *******************/
            /******************************************************************************/
            $bef_tax = get_post_meta($order->ID,'_cart_discount',true);
            if ($prod_opt == "1" && $order->order_discount == 0) {
                $order_products = $get_order->get_items();

                $order_products_datas = array ();
                //Per product datas
                foreach ($order_products as $item_id => $order_product) {
                    $order_product_id = $order_product['product_id'];
                    $order_product_acc = get_post_meta($order_product_id,'woocommerce_accounting_product_accounting_account',true);
                    $order_product_ana = get_post_meta($order_product_id,'woocommerce_accounting_product_accounting_analytic',true);
                    // Generic account applied if specific product account not set
                    if (!empty ($order_product_acc)){
                        $order_prod_acc = $order_product_acc;
                    }
                    else {
                        $order_prod_acc = $gen_account_prod;
                    }
                    // Generic analytic code applied if specific product code not set
                    if (!empty ($order_product_ana)) {
                        $order_prod_ana = $order_product_ana;
                    }
                    else {
                        $order_prod_ana = $gen_analytic_prod;
                    }
                    //  Creating the table
                    $order_products_datas[$item_id] = apply_filters('woocommerce_accounting:export:order_products_data', array (
                        'acc'=> $order_prod_acc,
                        'ana'=> $order_prod_ana,
                        'line_sub'=> $order_product['line_subtotal']
                    ), $order_product_id);
                }

                /******************************************************************************/
                /****************** Calcul des bons montants si code promo ********************/
                /******************************************************************************/

                //On choisit de faire apparaitre les codes promos comme des produits négatifs
                //Appliqué avant taxe (réduction et Code Promo) on raisonne en HT

                if ($bef_tax > 0) {
                    //Calculating ex.tax amount for coupons
                    // applied for each product
                    $bef_order_products_discounts = array();
                    foreach ($order_products as $key => $bef_order_product) {
                        $bef_order_product_id = $bef_order_product['product_id'];
                        $bef_discount_ht = ($bef_order_product['line_subtotal']) - ($bef_order_product['line_total']);
                        $bef_order_products_discounts[$key] = round($bef_discount_ht,2);
                    }
                }
            }

            /******** add exceptionnal income or outcome accounts ************/
            $exptcred_acc = get_option('woocommerce_accounting_generic_exptcred_accounting_account');
            $exptchar_acc = get_option('woocommerce_accounting_generic_exptchar_accounting_account');

            /*Different possibilities*/

            //Order amount
            $chk_tot = round($order->outcome,2);

            //Products
            if ($prod_opt == "0" OR $prod_opt == "2" OR $order->order_discount > 0) {
                $chk_prod = round($order->income_prodht,2);
                $chk_neg_prod = 0;
            }
            else {
                $col_prod = array();
                foreach ($order_products as $order_product) {
                    $col_prod[] = round($order_product['line_subtotal'],2) ;
                }
                $chk_prod = round(array_sum($col_prod),2);
                $chk_neg_prod = 0;
            }

            if ($chk_prod < 0) {
                $chk_neg_prod = abs($chk_prod);
                $chk_prod = 0;
            }

            //Taxes
            if ($taxes_opt == "0" OR $order->order_discount > 0) {
                $chk_tax = round($order->income_tax,2);
                $chk_neg_tax = 0;
            }
            else {
                $col_tax = array();
                foreach ($get_order->get_taxes() as $get_order_tax) {
                    $tax_line_am = $get_order_tax['tax_amount'] + $get_order_tax['shipping_tax_amount'];
                    $col_tax[] = round($tax_line_am,2) ;
                    $chk_neg_tax = 0;
                }
                $chk_tax = round(array_sum($col_tax),2);
                $chk_neg_tax = 0;
            }

            if ($chk_tax < 0) {
                $chk_neg_tax = abs($chk_tax);
                $chk_tax = 0;
            }

            //Shipment
            $chk_fdp = round($order->income_fdpht,2);

            //Coupon
            if ($bef_tax > 0 && $prod_opt == "1" && $order->order_discount == 0) {
                $chk_disc = round(array_sum($bef_order_products_discounts),2);
            }
            else {
                $chk_disc = 0;
            }

            // Difference
            $chk_op = ($chk_tot + $chk_disc + $chk_neg_prod + $chk_neg_tax) - ($chk_prod + $chk_tax + $chk_fdp);
            $chk_op = round($chk_op,2);
            $chk = abs($chk_op);

            //Export

            //Alert difference is more than a small rounding error
            if ($chk > 0.03) {
                // fputcsv ($output, array(
                //                             "ALERT !!! NEXT ORDER IS NOT MATCHING. YOU SHOULD CHECK IT."
                //                             ),
                //          $separator);
            }
            //Income
            if ($chk > 0 && $chk_op > 0) {
                $cred_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $exptcred_acc,
                    $col_map['Label'] =>    $order->lib.' ALERT!',
                    $col_map['Outcome'] =>    $fillwith,
                    $col_map['Income'] =>    str_replace('.',$dec_separator, round($chk,2)),
                    $col_map['Cost Center'] =>    "",
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );
                ksort($cred_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_cred_line', $cred_line, $order), $separator);
            }
            //Outcome
            elseif ($chk > 0 && $chk_op < 0) {
                $deb_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $exptchar_acc,
                    $col_map['Label'] =>    $order->lib.' ALERT!',
                    $col_map['Outcome'] =>    str_replace('.',$dec_separator, round($chk,2)),
                    $col_map['Income'] =>    $fillwith,
                    $col_map['Cost Center'] =>    "",
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );
                ksort($deb_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_deb_line', $deb_line, $order), $separator);
            }
            //Customer account
            if ((!empty ($order->account_cust)) && ($order->outcome != 0))  {
                $cust_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $order->account_cust,
                    $col_map['Label'] =>    $order->lib,
                    $col_map['Outcome'] =>    str_replace('.',$dec_separator, round($order->outcome,2)),
                    $col_map['Income'] =>    $fillwith,
                    $col_map['Cost Center'] =>    "",
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );
                ksort($cust_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_cust_line', $cust_line, $order), $separator);
            }
            else if ($order->outcome != 0) {
                $cust_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $book_account_status[$post_status] ? $book_account_status[$post_status] : $gen_account_cust,
                    $col_map['Label'] =>    $order->lib,
                    $col_map['Outcome'] =>    str_replace('.',$dec_separator, round($order->outcome,2)),
                    $col_map['Income'] =>    $fillwith,
                    $col_map['Cost Center'] =>    "",
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );
                ksort($cust_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_cust_line', $cust_line, $order), $separator);
            }
            //Shipping methods
            if ((!empty ($order_shipping_method)) && ($order->income_fdpht != 0)) {
                $deliv_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $order_shipping_accounting_acc,
                    $col_map['Label'] =>    $order->lib,
                    $col_map['Outcome'] =>    $fillwith,
                    $col_map['Income'] =>    str_replace('.',$dec_separator, round($order->income_fdpht,2)),
                    $col_map['Cost Center'] =>    $order_shipping_analytic_acc,
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );
                ksort($deliv_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_deliv_line', $deliv_line, $order),$separator);
            }
            //Products
            if ($prod_opt == "1" && $order->order_discount == 0) {
                foreach ($order_products_datas as $order_products_data) {
                    if ($order_products_data['line_sub'] != 0) {
                        if (round($order_products_data['line_sub'],2) > 0) {
                            $prod_inc = round($order_products_data['line_sub'],2);
                            $prod_out = "";
                        }
                        elseif (round($order_products_data['line_sub'],2) < 0 ){
                            $prod_inc = "";
                            $prod_out = abs(round($order_products_data['line_sub'],2));
                        }
                        $prod_line = array(
                            $col_map['Journal'] =>    $book_code,
                            $col_map['Date'] =>    $order->piecedate,
                            $col_map['Inv.Number'] =>    $order->number,
                            $col_map['Acc.Code'] =>    $order_products_data['acc'],
                            $col_map['Label'] =>    $order->lib,
                            $col_map['Outcome'] =>    str_replace('.',$dec_separator, $prod_out),
                            $col_map['Income'] =>    str_replace('.',$dec_separator, $prod_inc),
                            $col_map['Cost Center'] =>    $order_products_data['ana'],
                            $col_map['Empty 1'] =>    "",
                            $col_map['Empty 2'] =>    "",
                            $col_map['Empty 3'] =>    "",
                            $col_map['Empty 4'] =>    ""
                        );
                        ksort($prod_line);
                        fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_prod_line', $prod_line, $order_products_data, $order), $separator);
                    }
                }
                //Detailed coupon
                if ($bef_tax > 0) {
                    $coup_inc = 0;
                    $coup_out = 0;
                    foreach ($bef_order_products_discounts as $key => $bef_order_products_discount) {
                        // if (round($bef_order_products_discount,2) > 0) {
                        //     $coup_inc = $coup_inc + $fillwith;
                        //     $coup_out = $coup_out + round($bef_order_products_discount,2);
                        // } else if (round($bef_order_products_discount,2) < 0 ){
                        //     $coup_inc = $coup_inc + abs(round($bef_order_products_discount,2));
                        //     $coup_out = $coup_out + $fillwith;
                        // }
                        $coup_inc = $coup_inc + round($bef_order_products_discount,2);
                    }
                    $coupon_line = array(
                        $col_map['Journal'] =>    $book_code,
                        $col_map['Date'] =>    $order->piecedate,
                        $col_map['Inv.Number'] =>    $order->number,
                        $col_map['Acc.Code'] =>    $gen_account_discount,
                        $col_map['Label'] =>    $order->lib.' '.$gen_sku_discount,
                        $col_map['Outcome'] =>    '',
                        $col_map['Income'] =>    '-'.str_replace('.',$dec_separator,  $coup_inc),
                        $col_map['Cost Center'] =>    $gen_analytic_discount,
                        $col_map['Empty 1'] =>    "",
                        $col_map['Empty 2'] =>    "",
                        $col_map['Empty 3'] =>    "",
                        $col_map['Empty 4'] =>    ""
                    );
                    ksort($coupon_line);
                    fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_coupon_line', $coupon_line, $order),$separator);
                }
            }
            //Simple products
            elseif ($prod_opt == "0" && $order->income_prodht != 0) {
                if (round($order->income_prodht,2) > 0) {
                    $prod_inc = round($order->income_prodht,2);
                    $prod_out = $fillwith;
                }
                elseif (round($order->income_prodht,2) < 0 ){
                    $prod_inc = $fillwith;
                    $prod_out = abs(round($order->income_prodht,2));
                }
                $prod_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $gen_account_prod,
                    $col_map['Label'] =>    $order->lib,
                    $col_map['Outcome'] =>    str_replace('.',$dec_separator, $prod_out),
                    $col_map['Income'] =>    str_replace('.',$dec_separator, $prod_inc),
                    $col_map['Cost Center'] =>    $gen_analytic_prod,
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );

                ksort($prod_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_simple_prod_line', $prod_line, $order), $separator);
            }

            //Simple product by country
            elseif ($prod_opt == "2" && $order->income_prodht != 0) {
                $order->countryid = get_post_meta($order->ID,'_shipping_country',true);
                $order_country_acc = get_option('woocommerce_accounting_countries_acc');
                $country_account_prod = $order_country_acc[$order->countryid]['acc'];
                if (empty($country_account_prod)) {
                    $country_account_prod = $gen_account_prod;
                }
                if (round($order->income_prodht,2) > 0) {
                    $prod_inc = round($order->income_prodht,2);
                    $prod_out = $fillwith;
                }
                elseif (round($order->income_prodht,2) < 0 ){
                    $prod_inc = $fillwith;
                    $prod_out = abs(round($order->income_prodht,2));
                }
                $prod_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $country_account_prod,
                    $col_map['Label'] =>    $order->lib,
                    $col_map['Outcome'] =>    str_replace('.',$dec_separator, $prod_out),
                    $col_map['Income'] =>    str_replace('.',$dec_separator, $prod_inc),
                    $col_map['Cost Center'] =>    $gen_analytic_prod,
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );

                ksort($prod_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_country_prod_line', $prod_line, $order), $separator);
            }

            //Taxes
            if (($tax_applied_check != 0) && ($taxes_opt == "1") && ($order->order_discount == 0)) {
                foreach ($tax_displays as $tax_display) {
                    if (round($tax_display['amount'],2) > 0) {
                        $tax_inc = round($tax_display['amount'],2);
                        $tax_out = $fillwith;
                    }
                    elseif (round($tax_display['amount'],2) < 0 ){
                        $tax_inc = $fillwith;
                        $tax_out = abs(round($tax_display['amount'],2));
                    }
                    $tax_line = array(
                        $col_map['Journal'] =>    $book_code,
                        $col_map['Date'] =>    $order->piecedate,
                        $col_map['Inv.Number'] =>    $order->number,
                        $col_map['Acc.Code'] =>    $tax_display['account'],
                        $col_map['Label'] =>    $order->lib,
                        $col_map['Outcome'] =>    str_replace('.',$dec_separator, $tax_out),
                        $col_map['Income'] =>    str_replace('.',$dec_separator, $tax_inc),
                        $col_map['Cost Center'] =>    "",
                        $col_map['Empty 1'] =>    "",
                        $col_map['Empty 2'] =>    "",
                        $col_map['Empty 3'] =>    "",
                        $col_map['Empty 4'] =>    ""
                    );
                    ksort($tax_line);
                    fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_tax_line', $tax_line, $order), $separator);
                }
            }
            elseif ($tax_line_amount != 0){
                if (round($tax_line_amount,2) > 0) {
                    $tax_inc = round($tax_line_amount,2);
                    $tax_out = $fillwith;
                }
                elseif (round($tax_line_amount,2) < 0 ){
                    $tax_inc = $fillwith;
                    $tax_out = abs(round($tax_line_amount,2));
                }

                $tax_line = array(
                    $col_map['Journal'] =>    $book_code,
                    $col_map['Date'] =>    $order->piecedate,
                    $col_map['Inv.Number'] =>    $order->number,
                    $col_map['Acc.Code'] =>    $tax_line_account,
                    $col_map['Label'] =>    $order->lib,
                    $col_map['Outcome'] =>    str_replace('.',$dec_separator, $tax_out),
                    $col_map['Income'] =>    str_replace('.',$dec_separator, $tax_inc),
                    $col_map['Cost Center'] =>    "",
                    $col_map['Empty 1'] =>    "",
                    $col_map['Empty 2'] =>    "",
                    $col_map['Empty 3'] =>    "",
                    $col_map['Empty 4'] =>    ""
                );
                ksort($tax_line);
                fputcsv ($output, apply_filters('woocommerce_accounting:export:order_output_tax_line', $tax_line, $order), $separator);
            }

            do_action_ref_array('woocommerce_accounting:export:after_order_output', array(&$output, $order, $separator));
        }

        do_action_ref_array('woocommerce_accounting:export:before_close', array(&$output, $orders, $separator));
        fclose($output) or die("Can't close php://output");
        exit;
    }
    else {
        wp_safe_redirect(add_query_arg([
            'page'=>'woocommerce_accounting_exporter',
            'format'=>'basic',
            'error'=>'no_orders',
        ], admin_url('admin.php')));
        exit;
    }
}
